# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# Copyright IBM Corporation 2007,2008. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT
import sys

#--------------------------------------------------------------------------
# get the directory where WAS is installed
#--------------------------------------------------------------------------
def getWASHome():
	framework.log('info', 'RecoveryUpgrade', 'getWASHome', 'entering')
	ret = getPropertyValue( 'was.install.root' )
	message = 'exiting ' + ret
	framework.log('info', 'RecoveryUpgrade', 'getWASHome', message) 
	return ret

def getUserInstallRoot():
	framework.log('info', 'RecoveryUpgrade', 'getUserInstallRoot', 'entering')
	ret = getPropertyValue( 'user.install.root' )
	framework.log('info', 'RecoveryUpgrade', 'getUserInstallRoot', 'exiting ' + ret)
	return ret

#------------------------------------------------------------------------------
# get the property value by the the property name; If property not available, return ''
#------------------------------------------------------------------------------

def getPropertyValue( propertyName ):
	framework.log('info', 'RecoveryUpgrade', 'getPropertyValue', 'entering propertyName = ' + propertyName)
	propertyValue = java.lang.System.getProperty( propertyName )
	if( propertyValue == None ):
		framework.log('info', 'RecoveryUpgrade', 'getPropertyValue','exiting')
		return ''
	framework.log('info', 'RecoveryUpgrade', 'getPropertyValue','exiting ' + propertyValue)
	return propertyValue

#------------------------------------------------------------------------------
# a utility method
#------------------------------------------------------------------------------
def wsadminToList( inStr ):
	framework.log('info', 'RecoveryUpgrade', 'wsadminToList','entering ' + inStr)	
	outList = []
	if ( len( inStr ) > 0 and inStr[0] == '[' and inStr[-1] == ']' ):
		inStr = inStr[1:-1]
		tmpList = inStr.split( " " )
	else:
		tmpList = inStr.split( "\n" )	#splits for Windows or Linux
	for item in tmpList:
		item = item.rstrip();				#removes any Windows "\r"
		if ( len( item ) > 0 ):
			outList.append( item )
	framework.log('info', 'RecoveryUpgrade', 'wsadminToList','exiting ' + ''.join(outList))	
	return outList

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist( applicationName ):
	framework.log('info', 'RecoveryUpgrade', 'checkApplicationExist','entering ' + applicationName)
	exist = 0
	applications = AdminApp.list().split()
	for application in applications:
		if ( application == applicationName ):
			exist = 1
			framework.log('info', 'RecoveryUpgrade', 'checkApplicationExist','Exiting 1')
			return exist
	framework.log('info', 'RecoveryUpgrade', 'checkApplicationExist','Exiting 0')
	return exist

#------------------------------------------------------------------------------
# uninstall the application with the specified parameter
#------------------------------------------------------------------------------
def uninstallApplication( applicationName, node, server ):
	framework.log('info', 'RecoveryUpgrade', 'uninstallApplication','entering ' + applicationName)
	result = 0
	undeployOptions = ' -node ' + node + ' -server ' + server
	framework.log('info', 'RecoveryUpgrade', 'uninstallApplication', undeployOptions)
	try:
		AdminApp.uninstall( applicationName, undeployOptions )
	except:
		framework.log('error', 'RecoveryUpgrade', 'uninstallApplication', 'Error occurred when trying to uninstall application ' + applicationName)
		result = 1
	AdminConfig.save()
	framework.log('info', 'RecoveryUpgrade', 'uninstallApplication', 'exiting ' + str(result))
	return result

#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplication( oldAppName, applicationName, earLocation, cell, node, server ):
    #parameters = 'AppName ' + applicationName + ' location ' + earLocation + ' cell ' + cell + ' node ' + node + ' server ' + server
	framework.log('info', 'RecoveryUpgrade', 'installApplication', 'entering ')
	result = 0
	whetherApplicationExist = checkApplicationExist( applicationName )

	#1.IF: the application was not installed yet, install the application 
	if( whetherApplicationExist == 0 ):
		#deployOptions = '-appname ' + applicationName + ' -cell ' +  cell + ' -node ' + node + ' -server ' + server
		deployOptions = ['-appname', applicationName , '-cell', cell, '-node', node, '-server', server]
		dataSourceAttr = ['-DataSourceFor20EJBModules', [['wpsFEMgrEJB', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', 'jdbc/WPSDB']]]
		bindJndiAttr = ['-BindJndiForEJBMessageBinding', [['wpsFEMgrEJB', 'RecoveryMessageDrivenBean', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', [], 'manualrecovery', []]]]
		parameters = deployOptions + dataSourceAttr + bindJndiAttr
		try:
			AdminApp.install( earLocation, parameters )
		except:
			framework.log('error', 'RecoveryUpgrade', 'installApplication', 'Error occurred when trying to install application ' + applicationName)
			result = 1
		else:
			AdminConfig.save()

	#2.Else: the application was installed, add the mapping if available
	else:
		#2.1 Construct the new target
		newTarget = 'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server

		#2.2 Get the old target information
		moduleList = AdminApp.listModules( applicationName, '-server' )
		infoList = moduleList.split( '#' )
		module = 'wpsFEMgrEJB'
		uri = infoList[1].replace( '+', ',' )
		oldTarget = infoList[2]

		#2.3 check whether the target already exist in the target mapping
		result = checkTargetExist( oldTarget, newTarget )

		#2.3.1 If the target map is already defined, do nothing	
		if( result == 0 ):
			framework.log('warning', 'RecoveryUpgrade', 'installApplication', 'The application is already mapped to the new target '+ newTarget)

		#2.3.2 Otherwise, add the target mapping to the server
		else:
			#the moduleList is in format like 
			try:
				detailOption = [[module, uri, oldTarget + '+' + newTarget]]
				options = ['-MapModulesToServers', detailOption]
				AdminApp.edit( applicationName, options )
			except:
				framework.log('error', 'RecoveryUpgrade', 'installApplication', 'Error occurred while mapping target to server')
			else:
				AdminConfig.save()
	framework.log('info', 'RecoveryUpgrade', 'installApplication', 'exiting ' + str(result))
	return result;

#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplicationCluster( oldAppName, applicationName, earLocation, cell, cluster ):
	parameters = 'AppName ' + applicationName + ' location ' + earLocation + ' cell ' + cell + ' cluster ' + cluster
	framework.log('info', 'RecoveryUpgrade', 'installApplicationCluster','entering ' + parameters)
	result = 0
	whetherApplicationExist = checkApplicationExist( applicationName )

	#1.IF: the application was not installed yet, install the application 
	if( whetherApplicationExist == 0 ):
		deployOptions = ['-appname', applicationName , '-cell', cell, '-cluster', cluster]
		dataSourceAttr = ['-DataSourceFor20EJBModules', [['wpsFEMgrEJB', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', 'jdbc/WPSDB']]]
		bindJndiAttr = ['-BindJndiForEJBMessageBinding', [['wpsFEMgrEJB', 'RecoveryMessageDrivenBean', 'wpsFEMgrEJB.jar,META-INF/ejb-jar.xml', [], 'manualrecovery', []]]]
		parameters = deployOptions + dataSourceAttr + bindJndiAttr
		try:
			AdminApp.install( earLocation, parameters )
		except:
			framework.log('error', 'RecoveryUpgrade', 'installApplication', 'Error occurred when trying to install application ' + applicationName)
			return 1
		else:
			AdminConfig.save()

	#2.Else: the application was installed, add the mapping if available
	else:
		# 2.1 Construct the new target
		newTarget = 'WebSphere:cell=' + cell + ',cluster=' + cluster

		# 2.2 Get the old target information
		moduleList = AdminApp.listModules( applicationName, '-server' )
		infoList = moduleList.split( '#' )
		module = 'wpsFEMgrEJB'
		uri = infoList[1].replace( '+', ',' )
		oldTarget = infoList[2]

		#2.3 check whether the target already exist in the target mapping
		result = checkTargetExist( oldTarget, newTarget )

		#2.3.1 If the target map is already defined, do nothing	
		if( result == 0 ):
			framework.log('warning', 'RecoveryUpgrade', 'installApplication', 'The application is already mapped to the new target '+ newTarget)

		#2.3.2 Otherwise, add the target mapping to the server
		else:
			try:
				detailOption = [[module, uri, oldTarget + '+' + newTarget]]
				options = ['-MapModulesToServers', detailOption]
				AdminApp.edit( applicationName, options )
			except:
				framework.log('error', 'RecoveryUpgrade', 'installApplication', 'Error occurred while mapping target to server')
			else:
				AdminConfig.save()
	framework.log('info', 'RecoveryUpgrade', 'installApplication','exiting ' +str(result))
	return result;

#------------------------------------------------------------------------------
# check whether the newTarget already exist in oldTarget;If true,return 0; otherwise, return 1
#------------------------------------------------------------------------------
def checkTargetExist( oldTarget, newTarget ):
	framework.log('info', 'RecoveryUpgrade', 'checkTargetExist','entering ' + oldTarget + ' ' + newTarget)
	print "oldTarget = ", oldTarget
	print "newTarget = ", newTarget
	result = 1
	targetList = oldTarget.split( '+' )
	for target in targetList:
		if( target == newTarget ):
			result = 0
	framework.log('info', 'RecoveryUpgrade', 'checkTargetExist','exiting ' + str(result))
	return result

#------------------------------------------------------------------------------
# get the current cell
#------------------------------------------------------------------------------
def getCell():
	framework.log('info', 'RecoveryUpgrade', 'getCell','entering')
	cellName = '';
	cellList = wsadminToList( AdminConfig.list( 'Cell' ) )
	for cell in cellList:
		cellName = AdminConfig.showAttribute( cell, 'name' )
	framework.log('info', 'RecoveryUpgrade', 'getCell','exiting ' + cellName)
	return cellName


#------------------------------------------------------------------------------
# get the nodes defined in specified cell
#------------------------------------------------------------------------------
def getNodeList( cell ):
	framework.log('info', 'RecoveryUpgrade', 'getNodeList','entering ' + cell)
	nodeNameList = []
	cellId = AdminConfig.getid( '/Cell:' + cell )
	if( cellId == '' ):
		framework.log('info', 'RecoveryUpgrade', 'getNodeList','exiting ' + ''.join(nodeNameList))
		return nodeNameList
	nodeList = wsadminToList( AdminConfig.list( 'Node', cellId ) )
	for node in nodeList:
		nodeName = AdminConfig.showAttribute( node, 'name' )
		nodeNameList.append( nodeName )
	framework.log('info', 'RecoveryUpgrade', 'getNodeList','exiting '+ ''.join(nodeNameList))
	return nodeNameList

#------------------------------------------------------------------------------
# get the servers defined in specified cell and node; need to filter out the node agent, cluster member
#------------------------------------------------------------------------------
def getServerList( cell, node ):
	framework.log('info', 'RecoveryUpgrade', 'getServerList','entering cell ' + cell + ' node ' + node)
	serverNameList = []
	nodeId = AdminConfig.getid( '/Cell:' + cell + '/Node:' + node )
	if( nodeId == '' ):
		framework.log('info', 'RecoveryUpgrade', 'getServerList','exiting ' + ''.join(serverNameList))
		return serverNameList
	serverList = wsadminToList( AdminConfig.list( 'Server', nodeId ) )
	for server in serverList:
		serverName = AdminConfig.showAttribute( server, 'name' )
		clusterName = AdminConfig.showAttribute( server, 'clusterName' )
		if( clusterName == None ):
			if( serverName != 'nodeagent' ):
				serverNameList.append( serverName )
	framework.log('info', 'RecoveryUpgrade', 'getServerList','exiting ' + ''.join(serverNameList))
	return serverNameList

#------------------------------------------------------------------------------
# get the location of installable RemoteAL application
#------------------------------------------------------------------------------	
def getRecoveryApplicationLocation( cell, node ):
	framework.log('info', 'RecoveryUpgrade', 'getRecoveryApplicationLocation','entering cell ' + cell + ' node ' + node)
	type = 'wps'
	if ( profileType == 'ESB' ):
		type = 'esb'
	ret = getWASHome() + '/installableApps/wpsFEMgr_' + type + '.ear'
	framework.log('info', 'RecoveryUpgrade', 'getRecoveryApplicationLocation','exiting ' + ret)
	return ret

#------------------------------------------------------------------------------
# check whether the update is required for current node; If true, return 0; Otherwise, return 1
# 1. If the node is dmgr node, do nothing. The installation of Recovery application is done for the first custom node with server specified
# 
# 2. In the case of cluster upgrade, whether the node version info is available??
#------------------------------------------------------------------------------
def checkWhetherUpdateRequired():
	framework.log('info', 'RecoveryUpgrade', 'checkWhetherUpdateRequired','entering')
	result = 1
	nodeType = getPropertyValue( 'wbi.upgrade.node.type' )
	nodeVersion = getPropertyValue( 'wbi.upgrade.previous.version' )
	updatedCluster = getUpdatedCluster()

	if( updatedCluster != '' ):
		result = 0
	elif ( nodeType == 'managed' ) or ( nodeType == 'unmanaged' ):
		if( nodeVersion != '' ):
			result = 0
	framework.log('info', 'RecoveryUpgrade', 'checkWhetherUpdateRequired','exiting ' + str(result))
	return result

#------------------------------------------------------------------------------
# get the current node name which is migrated
#------------------------------------------------------------------------------
def getUpdatedNode():
	framework.log('info', 'RecoveryUpgrade', 'getUpdatedNode','entering')
	ret = getPropertyValue( 'local.node' )
	framework.log('info', 'RecoveryUpgrade', 'getUpdatedNode','exiting ' + ret)
	return ret

#------------------------------------------------------------------------------
# get the was.repository.root
#------------------------------------------------------------------------------
def getRepository():
	framework.log('info', 'RecoveryUpgrade', 'getRepository','entering')
	ret = getPropertyValue( 'was.repository.root' )
	framework.log('info', 'RecoveryUpgrade', 'getRepository','exiting  '+ret)
	return ret

#------------------------------------------------------------------------------
# get the current cluster name which is migrated, if not available return ''
#------------------------------------------------------------------------------
def getUpdatedCluster():
	framework.log('info', 'RecoveryUpgrade', 'getUpdatedCluster','entering')
	ret = getPropertyValue( 'wbi.upgrade.cluster.name' )
	framework.log('info', 'RecoveryUpgrade', 'getUpdatedCluster','exiting ' + ret)
	return ret

def checkOldAppMapping( oldAppName, cell, node, server, cluster ):
	parameters = 'oldAppName = ' + oldAppName + ' cell = ' + cell
	framework.log('info', 'RecoveryUpgrade', 'checkOldAppMapping','entering ' + parameters)
	wpsFEMgrModuleList = AdminApp.listModules( oldAppName, '-server' )
	wpsFEMgrInfoList = wpsFEMgrModuleList.split( '#' )
	wpsFEMgrModule = wpsFEMgrInfoList[0]
	wpsFEMgrURI = wpsFEMgrInfoList[1].replace( '+', ',' )
	wpsFEMgrTarget = wpsFEMgrInfoList[2]
	if ( cluster == None ):
		target = 'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server
	else:
		target = 'WebSphere:cell=' + cell + ',cluster=' + cluster
	result = checkTargetExist( wpsFEMgrTarget, target )
	if( result == 0 ):
		framework.log('info', 'RecoveryUpgrade', 'checkOldAppMapping','exiting 1')
		return 1
	else:
		framework.log('info', 'RecoveryUpgrade', 'checkOldAppMapping','exiting 0')
		return 0

def deleteOldMapping( oldAppName, cell, node, server, cluster ):
	parameters = 'oldAppName = ' + oldAppName + ' cell = ' + cell
	framework.log('info', 'RecoveryUpgrade', 'deleteOldMapping','entering ' + parameters)
	newTargetList = [];
	wpsFEMgrModuleList = AdminApp.listModules( oldAppName, '-server' )
	wpsFEMgrInfoList = wpsFEMgrModuleList.split( '#' )
	wpsFEMgrModule = 'FailedEventPersistenceEJB'
	wpsFEMgrURI = wpsFEMgrInfoList[1].replace( '+', ',' )
	position = wpsFEMgrURI.find( '.jar' )
	wpsFEMgrModule = wpsFEMgrURI[0:position]
	wpsFEMgrTarget = wpsFEMgrInfoList[2]
	wpsFEMgrTargetList = wpsFEMgrTarget.split( '+' )
	if( cluster == None ):
		oldTarget = 'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server
	else:
		oldTarget = 'WebSphere:cell=' + cell + ',cluster=' + cluster
	for target in wpsFEMgrTargetList:
		if ( target == oldTarget ):
			wpsFEMgrTargetList.remove( oldTarget )
			break
	if( wpsFEMgrTargetList == [] ):
		try:
			AdminApp.uninstall( oldAppName )
		except:
			framework.log('error', 'RecoveryUpgrade', 'deleteOldMapping','Error occurred when trying to uninstall the application ' + oldAppName)			
		else:
			AdminConfig.save()
		framework.log('info', 'RecoveryUpgrade', 'deleteOldMapping','exiting')
		return

	sep = '+'
	newTarget = sep.join( wpsFEMgrTargetList )
	try:
		detailOption = [[wpsFEMgrModule, wpsFEMgrURI, newTarget]]
		options = ['-MapModulesToServers', detailOption]
		AdminApp.edit( oldAppName, options )
	except:
		framework.log('error', 'RecoveryUpgrade', 'deleteOldMapping','Exception occurred when trying to edit target mapping')		
	else:
		AdminConfig.save()
	framework.log('info', 'RecoveryUpgrade', 'deleteOldMapping','exiting')

def deleteWebServerMapping( oldAppName ):
	framework.log('info', 'RecoveryUpgrade', 'deleteWebServerMapping','entering ' + oldAppName)
	if( checkApplicationExist( oldAppName ) == 0 ):
		framework.log('info', 'RecoveryUpgrade', 'deleteWebServerMapping', 'exiting, the application does not exist')		
		return
	wpsFEMgrModuleList = AdminApp.listModules( oldAppName, '-server' )
	framework.log('info', 'RecoveryUpgrade', 'deleteWebServerMapping', 'Failed event manager modules are ' + ''.join(wpsFEMgrModuleList))
	wpsFEMgrInfoList = wpsFEMgrModuleList.split( '#' )
	wpsFEMgrModule = 'FailedEventPersistenceEJB'
	wpsFEMgrURI = wpsFEMgrInfoList[1].replace( '+', ',' )
	position = wpsFEMgrURI.find( '.jar' )
	wpsFEMgrModule = wpsFEMgrURI[0:position]
	wpsFEMgrTarget = wpsFEMgrInfoList[2]
	oldTargetList = wpsFEMgrTarget.split( '+' )
	newTargetList = []
	for target in oldTargetList:
		if( isWebServerTarget( target ) == 0 ):
			newTargetList.append( target )
	framework.log('info', 'RecoveryUpgrade', 'deleteWebServerMapping', 'The target list is ' + ''.join(newTargetList))

	if( newTargetList == [] ):
		try:
			AdminApp.uninstall( oldAppName )
		except:
			framework.log('error', 'RecoveryUpgrade', 'deleteWebServerMapping','Error happened when trying to uninstall the application: ' + oldAppName)
		else:
			AdminConfig.save()
		return
	sep = '+'
	newTarget = sep.join( newTargetList )
	try:
		detailOption = [[wpsFEMgrModule, wpsFEMgrURI, newTarget]]
		options = ['-MapModulesToServers', detailOption]
		AdminApp.edit( oldAppName, options )
	except:
		framework.log('error', 'RecoveryUpgrade', 'deleteWebServerMapping','Exception happened when trying to edit target mapping')
	else:
		AdminConfig.save()
	framework.log('info', 'RecoveryUpgrade', 'deleteWebServerMapping','Exiting')

def isWebServerTarget( target ):
	framework.log('info', 'RecoveryUpgrade', 'isWebServerTarget','Entering ' + target)
	if( target.find( 'cluster=' ) != -1 ):
		framework.log('info', 'RecoveryUpgrade', 'isWebServerTarget','Exiting 0')
		return 0
	targetUnits = target.split( ',' )
	cellInfo = targetUnits[0]
	nodeInfo = targetUnits[1]
	serverInfo = targetUnits[2]
	cellName = cellInfo[15:len( cellInfo )]
	nodeName = nodeInfo[5:len( nodeInfo )]
	serverName = serverInfo[7:len( serverInfo )]
	nodeId = AdminConfig.getid( '/Cell:' + cellName + '/Node:' + nodeName )
	entriesStr = AdminConfig.list( 'ServerEntry', nodeId )
	entries = wsadminToList( entriesStr )
	for entry in entries:
		currentServerName = AdminConfig.showAttribute( entry, 'serverName' )
		if( currentServerName == serverName ):
			serverType = AdminConfig.showAttribute( entry, 'serverType' )
			if( serverType == 'WEB_SERVER' ):
				framework.log('info', 'RecoveryUpgrade', 'isWebServerTarget','Exiting 1')
				return 1
	framework.log('info', 'RecoveryUpgrade', 'isWebServerTarget','Exiting 0')
	return 0


def getDBType():
	framework.log('info', 'RecoveryUpgrade', 'getDBType','Entering')
	fileName = getUserInstallRoot() + '/properties/commondb.properties'
	file = open( fileName )
	lines = file.readlines();
	file.close()
	for line in lines:
		position = line.find( 'dbType=' )
		if( position >= 0 ):
			dbType = line[7:len( line )]
			dbType = dbType.strip()
			framework.log('info', 'RecoveryUpgrade', 'getDBType','Exiting ' + dbType)
			return dbType
	framework.log('info', 'RecoveryUpgrade', 'getDBType','Exiting')
	return ''

def removeMBeanProvider( nodeName, serverName, mbeanProviderName ):
	parameters = 'nodeName = ' + nodeName + ' serverName = ' + serverName + ' mbeanProviderName = ' + mbeanProviderName
	framework.log('info', 'RecoveryUpgrade', 'removeMBeanProvider','Entering ' + parameters)
	mbeanProviderIDListStr = AdminConfig.getid( '/ExtensionMBeanProvider:' + mbeanProviderName )
	mbeanProviderIDList = mbeanProviderIDListStr.split()
	for  mbeanProviderID in mbeanProviderIDList:
		if( ( mbeanProviderID == '' )or( mbeanProviderID.find( nodeName ) < 0 )or( mbeanProviderID.find( serverName ) < 0 ) ):
			continue
		try:
			AdminConfig.remove( mbeanProviderID )
		except:
			framework.log('error', 'RecoveryUpgrade', 'removeMBeanProvider','Error occur during remove MBean')
		else:
			AdminConfig.save()
			framework.log('info', 'RecoveryUpgrade', 'removeMBeanProvider','Exiting')
			return
	framework.log('info', 'RecoveryUpgrade', 'removeMBeanProvider','Exiting')

def removeCustomService( nodeName, serverName, className ):
	parameters = 'nodeName = ' + nodeName + ' serverName = ' + serverName + ' className = ' + className
	framework.log('info', 'RecoveryUpgrade', 'removeCustomService','Entering ' + parameters)
	serverIDListStr = AdminConfig.getid( '/Server:' + serverName )
	serverIDList = serverIDListStr.split()
	for serverID in serverIDList:
		if( ( serverID != '' )and ( serverID.find( nodeName ) > 0 ) ):
			elist = AdminConfig.showAttribute( serverID, 'customServices' )
			elista = elist.replace( '[', '' )
			elistb = elista.replace( ']', '' )
			elist = elistb.split()
			for e in elist:
				if( e == '' ):
					continue
				curClassName = AdminConfig.showAttribute( e, 'classname' )
				if( curClassName == className ):
					try:
						AdminConfig.remove( e )
					except:
						framework.log('error', 'RecoveryUpgrade', 'removeCustomService','Error occur during remove of custom service')
					else:
						AdminConfig.save()
						framework.log('info', 'RecoveryUpgrade', 'removeCustomService','Exiting')
						return
	framework.log('info', 'RecoveryUpgrade', 'removeCustomService','Exiting')

def getWBIVersion():
	framework.log('info', 'RecoveryUpgrade', 'getWBIVersion','Entering')
	from com.ibm.websphere.product import WASDirectory
	was = WASDirectory( getWASHome() )
	version = was.getVersion( 'BPMPS' )
	if( version == None or version == '' ):
		version = was.getVersion( 'BPMPC' )
	if( version == None or version == '' ):
		version = was.getVersion( 'ESB' )
	if(version == None or version == ''):
		framework.log('info', 'RecoveryUpgrade', 'getWBIVersion','Exiting version cannot be obtained')
		return version
	else:
		from java.lang import String
		version = String( version )
		version = version.substring( 0, 5 )
	framework.log('info', 'RecoveryUpgrade', 'getWBIVersion','Exiting ' + version)
	return version

def isVersionValid(version):
	framework.log('info', 'RecoveryUpgrade', 'isVersionValid','Entering ' + version)
	if(version.find('6.1.0') >= 0):
		return 1
	if(version.find('6.1.2') >= 0):
		return 1
	if(version.find('6.2.0') >= 0):
		return 1
	if(version.find('7.0.0') >= 0):
		return 1        
	framework.log('info', 'RecoveryUpgrade', 'isVersionValid','Exiting 0')
	return 0

def updateWPSProfile():
	framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile','Entering')
	nodeType = getPropertyValue( 'wbi.upgrade.node.type' )
	framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile','nodeType is ' + nodeType)
	previousVersion = getPropertyValue( 'wbi.upgrade.previous.version' )
	framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile','previous version is ' + previousVersion)
	prevVersion = previousVersion[:5]
	framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile','Truncated previous version is ' + prevVersion)
	if( nodeType == 'unmanaged' ):
		if( previousVersion.find( '6.0.1' ) >= 0 ):
			deleteWebServerMapping( 'wpsFEMgr' )
		elif( previousVersion.find( '6.0.2' ) >= 0 ):
			deleteWebServerMapping( 'wpsFEMgr_6.0.2.0' )
		else:
			feApplicationName = 'wpsFEMgr_' + prevVersion
			deleteWebServerMapping( feApplicationName )		
	elif( nodeType == 'dmgr' ):
		if( previousVersion.find( '6.0.1' ) >= 0 ):
			deleteWebServerMapping( 'wpsFEMgr' )
		elif( previousVersion.find( '6.0.2' ) >= 0 ):
			deleteWebServerMapping( 'wpsFEMgr_6.0.2.0' )
		else:
			feApplicationName = 'wpsFEMgr_' + prevVersion
			deleteWebServerMapping( feApplicationName )	
	# 1. check whether the update procedure should be invoked
	whetherUpdateRequired = checkWhetherUpdateRequired();

	# 2. If update required, perform update
	if ( whetherUpdateRequired == 0 ):
		#2.1 Initialization
		#recoveryApplicationName = 'wpsFEMgr_6.1.0'
		recoveryApplicationName = 'wpsFEMgr_' + getWBIVersion()
		cell = getCell()
		node = getUpdatedNode();
		recoveryAppLocation = getRecoveryApplicationLocation( cell, node )
		cluster = getUpdatedCluster()
		previousVersion = getPropertyValue( 'wbi.upgrade.previous.version' )

		##############################################################################
		#add for delete the nw wpsFEMgr_6.1.0 that is installed during creating standalone profile
		##############################################################################
		nodeType = getPropertyValue( 'wbi.upgrade.node.type' )
		if( nodeType == "unmanaged" ):
			if( checkApplicationExist( recoveryApplicationName ) == 1 ):
				try:
					AdminApp.uninstall( recoveryApplicationName )
				except:
					framework.log('error', 'RecoveryUpgrade', 'updateWPSProfile','Error happened when trying to uninstall the application ' + recoveryApplicationName)
				else:
					AdminConfig.save()

		#2.2 If node update is performed
		if( cluster == '' ):
			serverList = getServerList( cell, node )
			for server in serverList:
				#remove static MBean for adapting wps61
				removeMBeanProvider( node, server, 'recoveryMBeanProvider' )
				removeCustomService( node, server, 'com.ibm.wbiserver.management.FailedEventManagerMBean' )
				#check whether the server has the wpsFEMgr or wpsFEMgr_6.0.2.0  mapping
				prevVersion = previousVersion[:5]
				framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Truncated previous version is ' + prevVersion)
				if( previousVersion.find( '6.0.1' ) >= 0 ):
					whetheroldApplicationExist = checkApplicationExist( 'wpsFEMgr' )
					if ( whetheroldApplicationExist == 0 ):
						break
					oldMappingExist = checkOldAppMapping( 'wpsFEMgr', cell, node, server, None )
					if( oldMappingExist == 0 ):
						continue
					else:
						installApplication( 'wpsFEMgr', recoveryApplicationName, recoveryAppLocation, cell, node, server )
						deleteOldMapping( 'wpsFEMgr', cell, node, server, None )
				elif( previousVersion.find( '6.0.2' ) >= 0 ):
					whetheroldApplicationExist = checkApplicationExist( 'wpsFEMgr_6.0.2.0' )
					if ( whetheroldApplicationExist == 0 ):
						framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Old application wpsFEMgr_6.0.2 does not exist')
						break
					oldMappingExist = checkOldAppMapping( 'wpsFEMgr_6.0.2.0', cell, node, server, None )
					if( oldMappingExist == 0 ):
						continue
					else:
				 		installApplication( 'wpsFEMgr_6.0.2.0', recoveryApplicationName, recoveryAppLocation, cell, node, server )
				 		deleteOldMapping( 'wpsFEMgr_6.0.2.0', cell, node, server, None )
				else:
					versionValid = isVersionValid(prevVersion)					
					if (versionValid == 0):
						framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Version is not supported' + prevVersion)
					else:
						oldfeApplicationName = 'wpsFEMgr_' + prevVersion
						whetheroldApplicationExist = checkApplicationExist( oldfeApplicationName )
						if ( whetheroldApplicationExist == 0 ):
							framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Old application does not exist ' + oldfeApplicationName)
							break
						oldMappingExist = checkOldAppMapping( oldfeApplicationName, cell, node, server, None )
						if( oldMappingExist == 0 ):
							continue
						else:
				 			installApplication( oldfeApplicationName, recoveryApplicationName, recoveryAppLocation, cell, node, server )
				 			deleteOldMapping( oldfeApplicationName, cell, node, server, None )				

		#2.3 If cluster update is performed
		else:
			#check whether the cluster has the wpsFEMgr or wpsFEmgr_6.0.2.0 mapping.
			#If the cluster has the mapping ,delete the old mapping 

			clusterID = AdminConfig.getid( '/ServerCluster:' + cluster )
			memberListStr = AdminConfig.list( 'ClusterMember', clusterID )
			memberList = wsadminToList( memberListStr )
			for member in memberList:
				serverName = AdminConfig.showAttribute( member, 'memberName' )
				nodeName = AdminConfig.showAttribute( member, 'nodeName' )
				removeMBeanProvider( nodeName, serverName, 'recoveryMBeanProvider' )
				removeCustomService( nodeName, serverName, 'com.ibm.wbiserver.management.FailedEventManagerMBean' )

			if( previousVersion.find( '6.0.1' ) >= 0 ):
				whetheroldApplicationExist = checkApplicationExist( 'wpsFEMgr' )
				if ( whetheroldApplicationExist == 0 ):
					framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Old application wpsFEMgr does not exist')
				else:
					oldMappingExist = checkOldAppMapping( 'wpsFEMgr', cell, None, None, cluster )
					if( oldMappingExist == 0 ):
						framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'The old application wpsFEMgr_6.0.1 is not mapped to the target cluster')
					else:
						installApplicationCluster( 'wpsFEMgr', recoveryApplicationName, recoveryAppLocation, cell, cluster )
						deleteOldMapping( 'wpsFEMgr', cell, None, None, cluster )
			elif( previousVersion.find( '6.0.2' ) >= 0 ):
				 whetheroldApplicationExist = checkApplicationExist( 'wpsFEMgr_6.0.2.0' )
				 if ( whetheroldApplicationExist == 0 ):
					framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Old application wpsFEMgr_6.0.2 does not exist')
				 else:
					oldMappingExist = checkOldAppMapping( 'wpsFEMgr_6.0.2.0', cell, None, None, cluster )
					if( oldMappingExist == 0 ):
						framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'The old application wpsFEMgr_6.0.2 is not mapped to the target cluster')
					else:
						installApplicationCluster( 'wpsFEMgr_6.0.2.0', recoveryApplicationName, recoveryAppLocation, cell, cluster )
						deleteOldMapping( 'wpsFEMgr_6.0.2.0', cell, None, None, cluster )
			else:			
				prevVersion = previousVersion[:5]
				versionValid = isVersionValid(prevVersion)					
				if ( versionValid == 0 ):
					framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'The version is not supported ' + prevVersion)
				else:
					oldfeApplicationName = 'wpsFEMgr_' + prevVersion
					whetheroldApplicationExist = checkApplicationExist( oldfeApplicationName )
					if ( whetheroldApplicationExist == 0 ):
						framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'The failed event manager application did not exist')
					else:
						oldMappingExist = checkOldAppMapping( oldfeApplicationName, cell, None, None, cluster )
						if( oldMappingExist == 0 ):
							framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'The old failed event manager application is not mapped to the target cluster')
						else:
							installApplicationCluster( oldfeApplicationName, recoveryApplicationName, recoveryAppLocation, cell, cluster )
							deleteOldMapping( oldfeApplicationName, cell, None, None, cluster )
										
	# 3. Otherwise update not required, do nothing
	else:
		framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Nothing need to be done for Recovery component')
	framework.log('info', 'RecoveryUpgrade', 'updateWPSProfile', 'Exiting')
	
def findDestination( scabusName, queueId ):
	framework.log('info', 'RecoveryUpgrade', 'findDestination', 'Entering ' + scabusName + ' ' + queueId)

	busList = AdminConfig.list( 'SIBus' )
	busList = wsadminToList( busList )
	for bus in busList:
		busName = AdminConfig.showAttribute( bus, 'name' )
		if( busName == scabusName ):
			destinationList = AdminConfig.list( 'SIBQueue', bus )
			destinationList = wsadminToList( destinationList )
			for destination in destinationList:
				destinationID = AdminConfig.showAttribute( destination, 'identifier' )
				if( destinationID == queueId ):
					framework.log('info', 'RecoveryUpgrade', 'findDestination',	'The destination ' + destinationID + ' already exists')
					return destination
	framework.log('info', 'RecoveryUpgrade', 'findDestination',	'exiting ')
	
def modifySIBDestination( busName, nodeName, serverName, clusterName ):
	framework.log('info', 'RecoveryUpgrade', 'modifySIBDestination', 'Entering')
	parameters = busName + ' ' + nodeName + ' ' + serverName + ' ' + clusterName
	framework.log('info', 'RecoveryUpgrade', 'modifySIBDestination', parameters)
	
	if( clusterName == '' ):
		exceptionDestinationName = 'WBI.FailedEvent.'+nodeName+'.'+ serverName
		destName = 'sca/WLE_' + nodeName + '_' + serverName
	else:
		exceptionDestinationName = 'WBI.FailedEvent.'+ clusterName
		destName = 'sca/WLE_' + clusterName
	try:
		AdminTask.modifySIBDestination( '[-bus ' + busName + ' -name ' + destName + ' -exceptionDestination ' + exceptionDestinationName + ']' )
		AdminConfig.save()
	except:
		framework.log('error', 'RecoveryUpgrade', 'modifySIBDestination', 'Exiting 0, error occurred when trying to modifySIBDestination')
		return 0
	else:
		framework.log('info', 'RecoveryUpgrade', 'modifySIBDestination', 'Exiting 1, modified SIBDestination successful')
		return 1
	#framework.log('info', 'RecoveryUpgrade', 'modifySIBDestination', 'Exiting')
	
def updateWLESCANode():
	framework.log('info', 'RecoveryUpgrade', 'updateWLESCANode','Entering')
	cellName = getCell()
	busName = 'SCA.SYSTEM.' + cellName + '.Bus'
	nodeName = getUpdatedNode()
	serverList = getServerList(cellName, nodeName)
	for serverName in serverList:
		queueId = 'sca/WLE_' + nodeName + '_' + serverName
		destination = findDestination( busName, queueId)
		if( destination == None):
			framework.log('info', 'RecoveryUpgrade', 'updateWLESCANode','No WLE SCA destination found in node/server ' + nodeName+'/'+serverName)
		else:	
			modifySIBDestination( busName, nodeName, serverName, '')
			framework.log('info', 'RecoveryUpgrade', 'updateWLESCANode','WLE SCA destination modified in node/server ' + nodeName+'/'+serverName)
	framework.log('info', 'RecoveryUpgrade', 'updateWLESCANode','Exiting')
			
def updateWLESCACluster(cluster):
	framework.log('info', 'RecoveryUpgrade', 'updateWLESCACluster','Entering')
	cellName = getCell()
	busName = 'SCA.SYSTEM.' + cellName + '.Bus'
	queueId = 'sca/WLE_' + cluster
	destination = findDestination( busName, queueId)
	if( destination == None):
		framework.log('info', 'RecoveryUpgrade', 'updateWLESCACluster','No WLE SCA destination found in cluster ' + cluster)
	else:	
		modifySIBDestination( busName, '', '', cluster )
		framework.log('info', 'RecoveryUpgrade', 'updateWLESCACluster','WLE SCA destination modified in cluster ' + cluster)
	framework.log('info', 'RecoveryUpgrade', 'updateWLESCACluster','Exiting')

def updateWLESCADestination():
	framework.log('info', 'RecoveryUpgrade', 'updateWLESCADestination','Entering')
	nodeType = getPropertyValue( 'wbi.upgrade.node.type' )
	framework.log('info', 'RecoveryUpgrade', 'updateWLESCADestination','nodeType is ' + nodeType)
	# case1. node/profile is standalone
	if( nodeType == 'unmanaged' ):
		updateWLESCANode()		
	# case2. node/profile is managed
	else:		
		cluster = getUpdatedCluster()
		# case4. node/profile is not part of cluster, but a managed server in a cell
		if(cluster == ''):
			updateWLESCANode()
			framework.log('info', 'RecoveryUpgrade', 'updateWLESCADestination','Exiting: Updated managed servers in node')
		# case5. cluster is migrated	
		else:
			updateWLESCACluster(cluster)
			framework.log('info', 'RecoveryUpgrade', 'updateWLESCADestination','Exiting: Updated cluster')

#main 
framework = BPMMigrationFramework()
installDir = getWASHome()
framework.log('info', 'RecoveryUpgrade', 'RecoveryUpgrade', 'The install dir is ' + installDir)
execfile( installDir + '/util/Recovery/ESBRecoveryUpgrade.py' )
profileType = getPropertyValue( 'wbi.upgrade.profile.type' )
previousVersion = getPropertyValue( 'wbi.upgrade.previous.version' )
framework.log('info', 'RecoveryUpgrade', 'RecoveryUpgrade', 'Previous version is ' + previousVersion)
if ( profileType == 'ESB' ):
	framework.log('info', 'RecoveryUpgrade', 'RecoveryUpgrade', 'This is a ESB profile migration')
	if( previousVersion.find( '6.2.0' ) >= 0 ):
		#if it is a ESB6.2 then we do steps similar to WPS, because the failed event manager already exists
		updateWPSProfile()
	elif( previousVersion.find( '7.0.0' ) >= 0 ):
		#if it is a ESB7.0 then we do steps similar to WPS, because the failed event manager already exists
		updateWPSProfile()
	else:
		# if it is pre-6.2 then there was no recovery installed so this is a fresh install for ESB
		upgradeESBProfile()
elif ( profileType == 'WPS' ):
	framework.log('info', 'RecoveryUpgrade', 'RecoveryUpgrade', 'This is a WPS profile migration')
	updateWPSProfile()
	updateWLESCADestination()
else:
	framework.log('error', 'RecoveryUpgrade', 'RecoveryUpgrade', 'This profile is not supported ' + profileType)
	




